(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     30073,        754]*)
(*NotebookOutlinePosition[     30745,        777]*)
(*  CellTagsIndexPosition[     30701,        773]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Groebner Basis Methods for Multichannel Sampling with Unknown \
Offsets\n",
    FontSize->36],
  StyleBox["Luciano Sbaiz, Patrick Vandewalle, and Martin Vetterli",
    FontSize->18]
}], "Title"],

Cell["\<\
Code to reproduce Example 5.1 from the paper.
The paper, along with this code is available online at \
http://lcavwww.epfl.ch/reproducible_research/SbaizVV06 \
\>", "Abstract"],

Cell["\<\
Copyright (C) 2006 Laboratory of Audiovisual Communications (LCAV), \
      
Ecole Polytechnique Federale de Lausanne (EPFL),                          
CH-1015 Lausanne, Switzerland                                             
                                                                          
This program is free software; you can redistribute it and/or modify it under \
the terms of the GNU General Public License as published by the Free Software \
Foundation; either version 2 of the License, or (at your option) any later \
version. This software is distributed in the hope that it will be useful, but \
without any warranty; without even the implied warranty of merchantability or \
fitness for a particular purpose. See the GNU General Public License for more \
details (enclosed in the file GPL).\
\>", "Copyright"],

Cell["Latest modifications: June 13, 2006", "Text"],

Cell[CellGroupData[{

Cell["Define and initialize variables", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell["\<\
SeedRandom[7];
LL=5;
MM=2;
NN=4;
KK=(LL-1)/2;\
\>", "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[{
    \(\(y\  = {{y00, y01, y02, y03}, {y10, y11, y12, 
            y13}};\)\), "\[IndentingNewLine]", 
    \(\(X\  = {X0, X1, X2, X3, X4};\)\), "\[IndentingNewLine]", 
    \(\(z = {z1};\)\), "\[IndentingNewLine]", 
    \(\(vars\  = Join[X, z];\)\), "\[IndentingNewLine]", 
    \(\(z = Prepend[z, 1];\)\)}], "Input",
  CellLabel->"In[75]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["set the true coefficients and offsets as random \
parameters", "Text",
  FontFamily->"Helvetica",
  FontWeight->"Plain",
  FontVariations->{"CompatibilityType"->0}]], "Section",
  FontSize->18,
  FontWeight->"Plain"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Xtrue\  = \ Table[0, {i, LL}];\)\), "\[IndentingNewLine]", 
    \(\(Xtrue[\([KK + 1]\)] = 
        Random[Integer, {\(-10\), 10}];\)\), "\[IndentingNewLine]", 
    \(For[i = 1, 
      i < KK + 1, \(i++\), {Xtrue[\([KK + 1 + i]\)] = 
          Random[Integer, {\(-10\), 10}] + 
            I*Random[
                Integer, {\(-10\), 10}]; \[IndentingNewLine]Xtrue[\([KK + 1 - 
                i]\)] = 
          Conjugate[
            Xtrue[\([KK + 1 + 
                  i]\)]];\[IndentingNewLine]}]\), "\[IndentingNewLine]", 
    \(\(ttrue\  = \ Table[0, {i, MM}];\)\), "\[IndentingNewLine]", 
    \(\(For[i = 2, i < MM + 1, \(i++\), 
        ttrue[\([i]\)] = 
          Rational[Random[Integer, {1, 10}], 
            11]];\)\), "\[IndentingNewLine]", 
    \(\(ztrue = Exp[I*2*\[Pi]*ttrue];\)\), "\[IndentingNewLine]", 
    \({Xtrue, ttrue, ztrue}\), "\[IndentingNewLine]", 
    \(N[ztrue[\([2]\)]]\)}], "Input",
  CellLabel->"In[80]:="],

Cell[BoxData[
    \({{5 - \[ImaginaryI], \(-3\)\ \[ImaginaryI], \(-6\), 3\ \[ImaginaryI], 
        5 + \[ImaginaryI]}, {0, 
        6\/11}, {1, \[ExponentialE]\^\(-\(\(10\ \[ImaginaryI]\ \
\[Pi]\)\/11\)\)}}\)], "Output",
  CellLabel->"Out[86]="],

Cell[BoxData[
    \(\(-0.9594929736144973`\) - 
      0.28173255684143006`\ \[ImaginaryI]\)], "Output",
  CellLabel->"Out[87]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Construct the polynomials", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(Plist = Table[0, {i, MM*NN}]; 
    Plistmatrix = 
      Table[0, {i, MM*NN}, {j, LL + 1}];\), "\[IndentingNewLine]", 
    \(\(For[n = 1, 
        n < NN + 1, \(n++\), \[IndentingNewLine]\(For[m = 1, 
            m < MM + 
                1, \(m++\), {\[IndentingNewLine]Plist[\([n + 
                      NN*\((m - 1)\)]\)] = \(-\(y[\([m]\)]\)[\([n]\)]\) 
                  z[\([m]\)]^KK; 
              Plistmatrix[\([n + NN*\((m - 1)\), 
                    LL + 1]\)] = \(y[\([m]\)]\)[\([n]\)] 
                  z[\([m]\)]^KK; \[IndentingNewLine]For[l = 1, 
                l < LL + 
                    1, \(l++\), {\[IndentingNewLine]Plist[\([n + 
                          NN*\((m - 1)\)]\)] = 
                    Plist[\([n + NN*\((m - 1)\)]\)] + 
                      Exp[I*2*\[Pi]*\((n - 1)\) \((l - KK - 1)\)/NN] 
                        z[\([m]\)]^\((l - 1)\)*
                        X[\([l]\)]; \[IndentingNewLine]Plistmatrix[\([n + 
                          NN*\((m - 1)\), l]\)] = 
                    Exp[I*2*\[Pi]*\((n - 1)\) \((l - KK - 1)\)/NN] 
                      z[\([m]\)]^\((l - 
                            1)\);\[IndentingNewLine]}];\[IndentingNewLine]}\
\[IndentingNewLine]];\)\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(Plist\), "\[IndentingNewLine]", 
    \(Plistmatrix\)}], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \({X0 + X1 + X2 + X3 + X4 - y00, \(-X0\) - \[ImaginaryI]\ X1 + 
        X2 + \[ImaginaryI]\ X3 - X4 - y01, 
      X0 - X1 + X2 - X3 + X4 - y02, \(-X0\) + \[ImaginaryI]\ X1 + 
        X2 - \[ImaginaryI]\ X3 - X4 - y03, 
      X0 + X1\ z1 + X2\ z1\^2 - y10\ z1\^2 + X3\ z1\^3 + 
        X4\ z1\^4, \(-X0\) - \[ImaginaryI]\ X1\ z1 + X2\ z1\^2 - 
        y11\ z1\^2 + \[ImaginaryI]\ X3\ z1\^3 - X4\ z1\^4, 
      X0 - X1\ z1 + X2\ z1\^2 - y12\ z1\^2 - X3\ z1\^3 + 
        X4\ z1\^4, \(-X0\) + \[ImaginaryI]\ X1\ z1 + X2\ z1\^2 - 
        y13\ z1\^2 - \[ImaginaryI]\ X3\ z1\^3 - X4\ z1\^4}\)], "Output",
  CellLabel->"Out[90]="],

Cell[BoxData[
    \({{1, 1, 1, 1, 1, y00}, {\(-1\), \(-\[ImaginaryI]\), 
        1, \[ImaginaryI], \(-1\), y01}, {1, \(-1\), 1, \(-1\), 1, 
        y02}, {\(-1\), \[ImaginaryI], 1, \(-\[ImaginaryI]\), \(-1\), 
        y03}, {1, z1, z1\^2, z1\^3, z1\^4, 
        y10\ z1\^2}, {\(-1\), \(-\[ImaginaryI]\)\ z1, 
        z1\^2, \[ImaginaryI]\ z1\^3, \(-z1\^4\), y11\ z1\^2}, {1, \(-z1\), 
        z1\^2, \(-z1\^3\), z1\^4, y12\ z1\^2}, {\(-1\), \[ImaginaryI]\ z1, 
        z1\^2, \(-\[ImaginaryI]\)\ z1\^3, \(-z1\^4\), 
        y13\ z1\^2}}\)], "Output",
  CellLabel->"Out[91]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["compute sample values (samples are supposed to be real)", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(reprules = Table[0, {i, LL + MM - 1}];\)\), "\[IndentingNewLine]", 
    \(\(For[l = 1, l < LL + 1, \(l++\), 
        reprules[\([l]\)] = 
          X[\([l]\)] \[Rule] Xtrue[\([l]\)]];\)\), "\[IndentingNewLine]", 
    \(\(For[m = 1, m < MM, \(m++\), 
        reprules[\([LL + m]\)] = 
          z[\([m + 1]\)] \[Rule] 
            ztrue[\([m + 1]\)]];\)\), "\[IndentingNewLine]", 
    \(test := Plist\  /. reprules\), "\[IndentingNewLine]", 
    \(samples = Simplify[Solve[test \[Equal] 0]]\), "\[IndentingNewLine]", 
    \(realsamples = Re[y /. N[samples[\([1]\)]]]\)}], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \({{y00 \[Rule] 4, y01 \[Rule] \(-22\), y02 \[Rule] 4, 
        y03 \[Rule] \(-10\), 
        y10 \[Rule] \(-6\) + \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\^\(2/11\) \
+ 3\ \((\(-1\))\)\^\(9/22\) - 
            3\ \((\(-1\))\)\^\(13/22\) - \((5 - \[ImaginaryI])\)\ \
\((\(-1\))\)\^\(9/11\), 
        y11 \[Rule] \(-6\) + 
            3\ \((\(-1\))\)\^\(1/11\) - \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\
\^\(2/11\) + \((5 - \[ImaginaryI])\)\ \((\(-1\))\)\^\(9/11\) - 
            3\ \((\(-1\))\)\^\(10/11\), 
        y12 \[Rule] \(-6\) + \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\^\(2/11\) \
- 3\ \((\(-1\))\)\^\(9/22\) + 
            3\ \((\(-1\))\)\^\(13/22\) - \((5 - \[ImaginaryI])\)\ \
\((\(-1\))\)\^\(9/11\), 
        y13 \[Rule] \(-6\) - 
            3\ \((\(-1\))\)\^\(1/11\) + \((1 + 
                  5\ \[ImaginaryI])\)\ \((\(-1\))\)\^\(7/22\) - \((1 - 
                  5\ \[ImaginaryI])\)\ \((\(-1\))\)\^\(15/22\) + 
            3\ \((\(-1\))\)\^\(10/11\)}}\)], "Output",
  CellLabel->"Out[96]="],

Cell[BoxData[
    \({{4.`, \(-22.`\), 
        4.`, \(-10.`\)}, {3.021649034449195`, \(-7.574295851713632`\), \
\(-0.359141647647959`\), \(-19.088211535087595`\)}}\)], "Output",
  CellLabel->"Out[97]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["add Gaussian noise to the samples", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(<< Statistics`ContinuousDistributions`\), "\[IndentingNewLine]", 
    \(\(noise = 
        Table[Random[NormalDistribution[0, 1]], {i, MM}, {j, 
            NN}];\)\), "\[IndentingNewLine]", 
    \(noisysamples = N[realsamples + noise]\)}], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \({{3.4844899221494563`, \(-21.2467933966749`\), 
        3.667159566365723`, \(-9.531024276259005`\)}, {2.091677416681718`, \
\(-7.447950556525134`\), 
        0.729979434423164`, \(-19.30783212638691`\)}}\)], "Output",
  CellLabel->"Out[100]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert sample values into replacement rules", "Section"],

Cell[BoxData[{
    \(\(newsamples = Table[0, {i, MM}, {j, NN}];\)\), "\[IndentingNewLine]", 
    \(\(noiselesssamples = 
        Table[0, {i, MM}, {j, NN}];\)\), "\[IndentingNewLine]", 
    \(For[m = 1, 
      m < MM + 1, \(m++\), \[IndentingNewLine]For[n = 1, 
        n < NN + 
            1, \(n++\), {\[IndentingNewLine]noiselesssamples[\([m, n]\)] = 
            y[\([m, n]\)] \[Rule] realsamples[\([m, n]\)]; 
          newsamples[\([m, n]\)] = 
            y[\([m, n]\)] \[Rule] 
              noisysamples[\([m, 
                  n]\)];\[IndentingNewLine]}]]\), "\[IndentingNewLine]", 
    \(\(noiselesssamples = 
        Flatten[noiselesssamples];\)\), "\[IndentingNewLine]", 
    \(\(samples = Flatten[newsamples];\)\)}], "Input",
  CellLabel->"In[101]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["compute polynomials", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[BoxData[{
    \(\(polylist\  = Plist /. samples;\)\), "\[IndentingNewLine]", 
    \(\(polylistnoiseless = 
        Plist /. noiselesssamples;\)\), "\[IndentingNewLine]", 
    \(\(polylistmatrix = Plistmatrix /. {samples};\)\)}], "Input",
  CellLabel->"In[106]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["compute grobner basis", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell["\<\
first method: direct computation of a Grobner basis for all \
polynomials\
\>", "Subsubsection"],

Cell["gb = GroebnerBasis[polylist, vars]", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
second method: first eliminating all X, and computing a Grobner \
basis only for the remaining polynomials\
\>", "Subsubsection"],

Cell[BoxData[{
    \(\(coeff = Range[1, Length[polylist]];\)\), "\[IndentingNewLine]", 
    \(\(coefflists = 
        Subsets[coeff, {LL + MM - 1}];\)\), "\[IndentingNewLine]", 
    \(\(GroebnerSols = Table[0, {i, Length[coefflists]}];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(For[i = 1, i < Length[coefflists] + 1, \(i++\), 
      GroebnerSols[\([i]\)] = 
        GroebnerBasis[polylist[\([coefflists[\([i]\)]]\)], 
          vars]]\), "\[IndentingNewLine]", 
    \(GroebnerSols[\([1]\)]\)}], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \({\((\(\(0.031175972402324087`\)\(\[InvisibleSpace]\)\) + 
              0.9995139112312396`\ \[ImaginaryI])\)\ z1 + \
\((\(\(1.1191566295596707`\)\(\[InvisibleSpace]\)\) + 
              1.0847931391239976`\ \[ImaginaryI])\)\ z1\^2 + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^3, \((\(-9.482366790362272`\) + 
            6.211112542963891`*^-16\ \[ImaginaryI])\) + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ X4 - \((\(\(11.504256337733233`\)\(\
\[InvisibleSpace]\)\) + 
              8.266343267209738`\ \[ImaginaryI])\)\ z1 - \
\((\(\(8.536261611918802`\)\(\[InvisibleSpace]\)\) + 
              13.25820804569055`\ \[ImaginaryI])\)\ z1\^2, \
\((\(\(0.04566741105406624`\)\(\[InvisibleSpace]\)\) - 
            2.928942280103973`\ \[ImaginaryI])\) + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ X3, \((\(\(5.906542046104681`\)\(\
\[InvisibleSpace]\)\) + 
            0.`\ \[ImaginaryI])\) + \((\(\(1.`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ X2, \((\(\(0.04566741105406624`\)\(\
\[InvisibleSpace]\)\) + 
            2.928942280103975`\ \[ImaginaryI])\) + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI])\)\ X1, 
      1.`\ X0 + \((\(\(11.504256337733233`\)\(\[InvisibleSpace]\)\) + 
              8.266343267209738`\ \[ImaginaryI])\)\ z1 + \
\((\(\(8.536261611918802`\)\(\[InvisibleSpace]\)\) + 
              13.25820804569055`\ \[ImaginaryI])\)\ z1\^2}\)], "Output",
  CellLabel->"Out[113]="]
}, Open  ]],

Cell[BoxData[{
    \(\(AllVarssolved = 
        Table[0, {i, Length[GroebnerSols]}];\)\), "\[IndentingNewLine]", 
    \(For\ [i = 1, 
      i < Length[GroebnerSols] + 
          1, \(i++\), {\(AllVarssolved[\([i]\)] = 
            Solve[GroebnerSols[\([i]\)] \[Equal] 0, 
              vars];\)\[IndentingNewLine]}]\), "\[IndentingNewLine]", 
    \(\(AllVarssolved = Flatten[N[AllVarssolved], 1];\)\)}], "Input",
  CellLabel->"In[114]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["eliminate shifts that do not give a solution", \
"Subsubsection",
  FontFamily->"Helvetica",
  FontSize->16]], "Section"],

Cell["\<\

- shifts not on the unit circle
- offset vectors that are all 1 (or -1)
- offset vectors that do not give a solution to the equations from \
polylistmatrix\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(zsolved = {};\)\), "\[IndentingNewLine]", 
    \(For[i = 1, 
      i < Length[GroebnerSols] + 1, \(i++\), {\[IndentingNewLine]Temp = 
          z /. AllVarssolved[\([i]\)]; \[IndentingNewLine]zabs = 
          Abs[Temp]; \[IndentingNewLine]onecount = 
          Table[0, {i, MM - 1}]; \[IndentingNewLine]For[k = 1, 
          k < MM, \(k++\), {\[IndentingNewLine]ztmp = 
              1; \[IndentingNewLine]onecount[\([k]\)] = 
              0; \[IndentingNewLine]For[m = 1, 
              m < MM + 1, \(m++\), {\[IndentingNewLine]If[
                  zabs[\([m]\)] > 1 + 10\^\(-5\) || 
                    zabs[\([m]\)] < 1 - 10\^\(-5\), 
                  ztmp = 0]; \[IndentingNewLine]If[
                  Abs[Re[Temp[\([k, m]\)]]] < 1 + 10\^\(-5\) && 
                    Abs[Re[Temp[\([k, m]\)]]] > 1 - 10\^\(-5\), 
                  onecount[\([k]\)] = 
                    onecount[\([k]\)] + 
                      1];\[IndentingNewLine]}]; \[IndentingNewLine]If[
              ztmp \[Equal] 1 && onecount[\([k]\)] \[NotEqual] MM && 
                Length[AllVarssolved[\([i]\)]] == 
                  LL + MM - 1, \[IndentingNewLine]{If[
                  Norm[polylist /. AllVarssolved[\([i]\)]] < 1, 
                  zsolved = 
                    Append[zsolved, 
                      AllVarssolved[\([i]\)]]]}];\[IndentingNewLine]}]\
\[IndentingNewLine]}]\), "\[IndentingNewLine]", 
    \(zsolved\)}], "Input"],

Cell[BoxData[
    \({{X0 \[Rule] \(\(4.741183395181137`\)\(\[InvisibleSpace]\)\) - 
            5.308646607155946`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406624`\) - 
            2.928942280103975`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.906542046104681`\), 
        X3 \[Rule] \(-0.045667411054066236`\) + 
            2.928942280103973`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.741183395181136`\)\(\[InvisibleSpace]\)\) + 
            5.308646607155943`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9957181720804655`\) - 
            0.09244090971391783`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.741183395181149`\)\(\[InvisibleSpace]\)\) - 
            4.921565456936027`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406624`\) - 
            2.928942280103975`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.906542046104681`\), 
        X3 \[Rule] \(-0.045667411054066236`\) + 
            2.928942280103973`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.741183395181121`\)\(\[InvisibleSpace]\)\) + 
            4.921565456936026`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9949152873742123`\) - 
            0.10071529650003008`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.7411833951811655`\)\(\[InvisibleSpace]\)\) - 
            7.8284042620149235`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406624`\) - 
            2.928942280103975`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.906542046104681`\), 
        X3 \[Rule] \(-0.045667411054066236`\) + 
            2.928942280103973`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.741183395181115`\)\(\[InvisibleSpace]\)\) + 
            7.828404262014921`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9982350811929285`\) - 
            0.05938621621005924`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.7411833951811495`\)\(\[InvisibleSpace]\)\) - 
            4.473648649938119`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406624`\) - 
            2.928942280103975`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.906542046104681`\), 
        X3 \[Rule] \(-0.045667411054066236`\) + 
            2.928942280103973`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.741183395181125`\)\(\[InvisibleSpace]\)\) + 
            4.4736486499381165`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9942300811737118`\) - 
            0.1072685680398271`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.741183395181129`\)\(\[InvisibleSpace]\)\) - 
            2.316983183085607`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406624`\) - 
            2.928942280103975`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.906542046104681`\), 
        X3 \[Rule] \(-0.045667411054066236`\) + 
            2.928942280103973`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.74118339518114`\)\(\[InvisibleSpace]\)\) + 
            2.3169831830856045`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9881200721231143`\) - 
            0.15368384126970458`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.746532675438106`\)\(\[InvisibleSpace]\)\) - 
            4.899682940295098`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.0456674110540676`\) - 
            2.9182437195900546`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.917240606618601`\) - 
            1.5489868186805654`*^-15\ \[ImaginaryI], 
        X3 \[Rule] \(-0.04566741105406487`\) + 
            2.9182437195900546`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.746532675438086`\)\(\[InvisibleSpace]\)\) + 
            4.8996829402951`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9948777811655037`\) - 
            0.10108511533952186`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.719282860951882`\)\(\[InvisibleSpace]\)\) - 
            7.950476047970039`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.04566741105406762`\) - 
            2.972743348562581`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.862740977646076`\) - 
            9.855536462549394`*^-16\ \[ImaginaryI], 
        X3 \[Rule] \(-0.04566741105406579`\) + 
            2.9727433485625814`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.719282860951783`\)\(\[InvisibleSpace]\)\) + 
            7.95047604797004`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.998284658424371`\) - 
            0.058546910717282734`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.7029103450072745`\)\(\[InvisibleSpace]\)\) - 
            2.435499481546107`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.045667411054066576`\) - 
            3.005488380451674`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.829995945756981`\) - 
            2.0040247693569022`*^-17\ \[ImaginaryI], 
        X3 \[Rule] \(-0.045667411054066076`\) + 
            3.0054883804516725`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.702910345007298`\)\(\[InvisibleSpace]\)\) + 
            2.435499481546106`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9887876629813404`\) - 
            0.14932835476191073`\ \[ImaginaryI]}, {X0 \[Rule] \
\(\(4.730576686877855`\)\(\[InvisibleSpace]\)\) - 
            4.431986051904895`\ \[ImaginaryI], 
        X1 \[Rule] \(-0.024453994447514293`\) - 
            2.9289422801039695`\ \[ImaginaryI], 
        X2 \[Rule] \(-5.927755462711237`\) - 
            3.2050409978780962`*^-15\ \[ImaginaryI], 
        X3 \[Rule] \(-0.024453994447513994`\) + 
            2.9289422801039757`\ \[ImaginaryI], 
        X4 \[Rule] \(\(4.730576686877862`\)\(\[InvisibleSpace]\)\) + 
            4.431986051904894`\ \[ImaginaryI], 
        z1 \[Rule] \(-0.9941587701679087`\) - 
            0.10792747425114194`\ \[ImaginaryI]}}\)], "Output",
  CellLabel->"Out[119]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["compute the average of the offsets", "Subsubsection",
  FontFamily->"Helvetica",
  FontSize->16]], "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(zVarsolved = z /. zsolved;\)\), "\[IndentingNewLine]", 
    \(\(offsetsolved = 
        Re[Log[zVarsolved]/\((2  \[Pi]*I)\)];\)\), "\[IndentingNewLine]", 
    \(\(tSolved = Mean[offsetsolved];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<t = \>", 
        NN*\((tSolved[\([2]\)] + 1)\)];\)\), "\[IndentingNewLine]", 
    \(zSolvedAvgRule = Table[0, {i, MM - 1}]; 
    For[i = 2, i < MM + 1, \(i++\), 
      zSolvedAvgRule[\([i - 1]\)] = 
        z[\([i]\)] \[Rule] 
          N[Exp[I*2*\[Pi]*tSolved[\([i]\)]]]];\), "\[IndentingNewLine]", 
    \(zSolvedAvgRule\)}], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    InterpretationBox[\("t = "\[InvisibleSpace]2.0659620736600592`\),
      SequenceForm[ "t = ", 2.0659620736600592],
      Editable->False]], "Print",
  CellLabel->"From In[120]:="],

Cell[BoxData[
    \({z1 \[Rule] \(-0.9946369754151528`\) - 
          0.10342769037833421`\ \[ImaginaryI]}\)], "Output",
  CellLabel->"Out[125]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["compute the solution", "Subsubsection",
  FontFamily->"Helvetica",
  FontSize->16]], "Section",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(CoeffMatrix = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {1, LL}] /. 
          zSolvedAvgRule;\)\), "\[IndentingNewLine]", 
    \(\(SampleVector = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {LL + 1, LL + 1}] /. 
          zSolvedAvgRule;\)\), "\[IndentingNewLine]", 
    \(Xcomputed = 
      PseudoInverse[CoeffMatrix] . SampleVector\), "\[IndentingNewLine]", 
    \(XcomputedRule = Table[0, {i, LL}]; 
    For[i = 1, i < LL + 1, \(i++\), 
      XcomputedRule[\([i]\)] = 
        X[\([i]\)] \[Rule] Xcomputed[\([i]\)]];\)}], "Input",
  CellLabel->"In[126]:="],

Cell[BoxData[
    \({{\(\(4.741183395181132`\)\(\[InvisibleSpace]\)\) - 
          4.581220563762119`\ \[ImaginaryI]}, {\(-0.03880308097342678`\) - 
          2.95661039139286`\ \[ImaginaryI]}, {\(-5.945036752028229`\) + 
          1.511587788052522`*^-14\ \[ImaginaryI]}, \
{\(-0.038803080973431756`\) + 
          2.956610391392887`\ \[ImaginaryI]}, {\(\(4.741183395181137`\)\(\
\[InvisibleSpace]\)\) + 4.581220563762124`\ \[ImaginaryI]}}\)], "Output",
  CellLabel->"Out[128]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compute the offsets t from the z, and display solutions with the \
norm of the error\
\>", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(TfromZ = 
        N[Log[z /. 
                zSolvedAvgRule]/\((I*2*\[Pi])\)];\)\), "\[IndentingNewLine]", \

    \(For[j = 1, j < MM + 1, \(j++\), 
      If[Im[TfromZ[\([j]\)]] < 
          10\^\(-10\), \(TfromZ[\([j]\)] = 
            Re[TfromZ[\([j]\)]];\)]]\), "\[IndentingNewLine]", 
    \(\(CoeffMatrixExactOffsets = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {1, LL}] /. 
          Take[reprules, \(-\((MM - 1)\)\)];\)\), "\[IndentingNewLine]", 
    \(\(SampleVectorExactOffsets = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {LL + 1, LL + 1}] /. 
          Take[reprules, \(-\((MM - 1)\)\)];\)\), "\[IndentingNewLine]", 
    \(\(\(XcomputedExactOffsets = 
        PseudoInverse[CoeffMatrixExactOffsets] . 
          SampleVectorExactOffsets;\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(Print["\<Solution: \>"];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Exact offsets:\>", 
        N[ttrue], "\<\n\>", \[IndentingNewLine]"\<Estimated offsets:\>", 
        TfromZ];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Exact coefficients:\>", 
        Xtrue, "\<\n\n\>", "\<Estimated coefficients with exact offsets:\>", 
        Flatten[XcomputedExactOffsets], "\<\n\>", "\<Coefficient error using \
exact shifts: \>", 
        Norm[XcomputedExactOffsets - Xtrue]/
          Norm[Xtrue], "\<\n\n\>", "\<Estimated coefficients with estimated \
shifts:\>", 
        Xcomputed, "\<\n\>", "\<Coefficient error using estimated shifts: \
\>", Norm[Flatten[Xcomputed] - Xtrue]/
          Norm[Xtrue]];\)\), "\[IndentingNewLine]", 
    \(\(Print[
        Norm[\(polylistnoiseless /. XcomputedRule\) /. 
            zSolvedAvgRule]];\)\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \("Solution: "\)], "Print",
  CellLabel->"From In[130]:="],

Cell[BoxData[
    InterpretationBox[\("Exact offsets:"\[InvisibleSpace]{0.`, 
          0.5454545454545454`}\[InvisibleSpace]"\n\
"\[InvisibleSpace]"Estimated offsets:"\[InvisibleSpace]{0.`, \
\(-0.48350948158498525`\)}\),
      SequenceForm[ 
      "Exact offsets:", {0.0, 0.54545454545454541}, "\n", 
        "Estimated offsets:", {0.0, -0.48350948158498525}],
      Editable->False]], "Print",
  CellLabel->"From In[130]:="],

Cell[BoxData[
    InterpretationBox[\("Exact coefficients:"\[InvisibleSpace]{5 - \
\[ImaginaryI], \(-3\)\ \[ImaginaryI], \(-6\), 3\ \[ImaginaryI], 
          5 + \[ImaginaryI]}\[InvisibleSpace]"\n\n\
"\[InvisibleSpace]"Estimated coefficients with exact offsets:"\
\[InvisibleSpace]{\(\(4.741183395181135`\)\(\[InvisibleSpace]\)\) - 
            0.5389111029680582`\ \[ImaginaryI], \(\(0.23151318351691413`\)\(\
\[InvisibleSpace]\)\) - 
            2.934859601098233`\ \[ImaginaryI], \(-5.945036752028237`\) - 
            1.0375309099882165`*^-16\ \[ImaginaryI], \(\(0.23151318351691505`\
\)\(\[InvisibleSpace]\)\) + 
            2.9348596010982324`\ \[ImaginaryI], \(\(4.741183395181135`\)\(\
\[InvisibleSpace]\)\) + 
            0.5389111029680582`\ \[ImaginaryI]}\[InvisibleSpace]"\n"\
\[InvisibleSpace]"Coefficient error using exact shifts: \
"\[InvisibleSpace]0.07996946480000004`\[InvisibleSpace]"\n\n\
"\[InvisibleSpace]"Estimated coefficients with estimated shifts:"\
\[InvisibleSpace]{{\(\(4.741183395181132`\)\(\[InvisibleSpace]\)\) - 
              4.581220563762119`\ \[ImaginaryI]}, {\(-0.03880308097342678`\) \
- 2.95661039139286`\ \[ImaginaryI]}, {\(-5.945036752028229`\) + 
              1.511587788052522`*^-14\ \[ImaginaryI]}, \
{\(-0.038803080973431756`\) + 
              2.956610391392887`\ \[ImaginaryI]}, {\(\(4.741183395181137`\)\(\
\[InvisibleSpace]\)\) + 
              4.581220563762124`\ \[ImaginaryI]}}\[InvisibleSpace]"\n"\
\[InvisibleSpace]"Coefficient error using estimated shifts: \
"\[InvisibleSpace]0.49329515023079057`\),
      SequenceForm[ "Exact coefficients:", {
        Complex[ 5, -1], 
        Complex[ 0, -3], -6, 
        Complex[ 0, 3], 
        Complex[ 5, 1]}, "\n\n", 
        "Estimated coefficients with exact offsets:", {
        Complex[ 4.7411833951811353, -0.53891110296805822], 
        Complex[ 0.23151318351691413, -2.9348596010982329], 
        Complex[ -5.945036752028237, -1.0375309099882165*^-16], 
        Complex[ 0.23151318351691505, 2.9348596010982324], 
        Complex[ 4.7411833951811353, 0.53891110296805822]}, "\n", 
        "Coefficient error using exact shifts: ", 0.079969464800000042, 
        "\n\n", "Estimated coefficients with estimated shifts:", {{
        Complex[ 4.7411833951811317, -4.5812205637621188]}, {
        Complex[ -0.03880308097342678, -2.9566103913928599]}, {
        Complex[ -5.945036752028229, 1.5115877880525221*^-14]}, {
        Complex[ -0.038803080973431756, 2.9566103913928869]}, {
        Complex[ 4.7411833951811371, 4.5812205637621242]}}, "\n", 
        "Coefficient error using estimated shifts: ", 0.49329515023079057],
      Editable->False]], "Print",
  CellLabel->"From In[130]:="],

Cell[BoxData[
    \(1.7811792852265935`\)], "Print",
  CellLabel->"From In[130]:="]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 793}},
WindowSize->{699, 771},
WindowMargins->{{96, Automatic}, {Automatic, 0}},
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 222, 6, 144, "Title"],
Cell[1979, 59, 186, 4, 36, "Abstract"],
Cell[2168, 65, 843, 13, 138, "Copyright"],
Cell[3014, 80, 51, 0, 26, "Text"],

Cell[CellGroupData[{
Cell[3090, 84, 94, 1, 59, "Section"],
Cell[3187, 87, 95, 7, 107, "Input"],
Cell[3285, 96, 354, 7, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3676, 108, 240, 6, 60, "Section"],

Cell[CellGroupData[{
Cell[3941, 118, 964, 21, 239, "Input"],
Cell[4908, 141, 245, 5, 64, "Output"],
Cell[5156, 148, 128, 3, 47, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5333, 157, 88, 1, 59, "Section"],

Cell[CellGroupData[{
Cell[5446, 162, 1378, 27, 287, "Input"],
Cell[6827, 191, 642, 11, 119, "Output"],
Cell[7472, 204, 575, 10, 99, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8096, 220, 118, 1, 59, "Section"],

Cell[CellGroupData[{
Cell[8239, 225, 627, 12, 127, "Input"],
Cell[8869, 239, 1019, 20, 119, "Output"],
Cell[9891, 261, 202, 4, 47, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10142, 271, 52, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[10219, 275, 295, 6, 79, "Input"],
Cell[10517, 283, 264, 5, 60, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10830, 294, 63, 0, 59, "Section"],
Cell[10896, 296, 767, 16, 175, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11700, 317, 82, 1, 59, "Section"],
Cell[11785, 320, 268, 5, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12090, 330, 84, 1, 59, "Section"],

Cell[CellGroupData[{
Cell[12199, 335, 106, 3, 43, "Subsubsection"],
Cell[12308, 340, 50, 0, 26, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12395, 345, 139, 3, 43, "Subsubsection"],
Cell[12537, 350, 251, 4, 79, "Input"],

Cell[CellGroupData[{
Cell[12813, 358, 272, 6, 79, "Input"],
Cell[13088, 366, 1559, 28, 132, "Output"]
}, Open  ]],
Cell[14662, 397, 438, 9, 111, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15149, 412, 145, 3, 61, "Section"],
Cell[15297, 417, 180, 6, 110, "Text"],

Cell[CellGroupData[{
Cell[15502, 427, 1452, 27, 384, "Input"],
Cell[16957, 456, 5377, 104, 340, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22383, 566, 133, 2, 61, "Section"],

Cell[CellGroupData[{
Cell[22541, 572, 624, 13, 159, "Input"],
Cell[23168, 587, 197, 4, 23, "Print"],
Cell[23368, 593, 146, 3, 60, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[23563, 602, 135, 3, 61, "Section"],

Cell[CellGroupData[{
Cell[23723, 609, 599, 13, 143, "Input"],
Cell[24325, 624, 480, 8, 78, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[24854, 638, 111, 3, 59, "Section"],

Cell[CellGroupData[{
Cell[24990, 645, 1765, 37, 439, "Input"],
Cell[26758, 684, 76, 2, 23, "Print"],
Cell[26837, 688, 427, 9, 39, "Print"],
Cell[27267, 699, 2692, 47, 171, "Print"],
Cell[29962, 748, 83, 2, 23, "Print"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

