(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37072,        905]*)
(*NotebookOutlinePosition[     37744,        928]*)
(*  CellTagsIndexPosition[     37700,        924]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Groebner Basis Methods for Multichannel Sampling with Unknown \
Offsets\n",
    FontSize->36],
  StyleBox["Luciano Sbaiz, Patrick Vandewalle, and Martin Vetterli",
    FontSize->18]
}], "Title"],

Cell["\<\
Code to reproduce Example 6.1 from the paper.
The paper, along with this code is available online at \
http://lcavwww.epfl.ch/reproducible_research/SbaizVV06 \
\>", "Abstract"],

Cell["\<\
Copyright (C) 2006 Laboratory of Audiovisual Communications \
(LCAV),Ecole Polytechnique Federale de Lausanne (EPFL),CH-1015 \
Lausanne,Switzerland

This program is free software;you can redistribute it and/or modify it under \
the terms of the GNU General Public License as published by the Free Software \
Foundation;either version 2 of the License,or (at your option) any later \
version.This software is distributed in the hope that it will be useful,but \
without any warranty;without even the implied warranty of merchantability or \
fitness for a particular purpose.See the GNU General Public License for more \
details (enclosed in the file GPL).\
\>", "Copyright"],

Cell["Latest modifications: June 13, 2006", "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Define and initialize variables",
  FontVariations->{"CompatibilityType"->0}]], "Section"],

Cell["\<\
SeedRandom[7];
LL=5;
MM=2;
NN=4;
KK=(LL-1)/2;\
\>", "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[{
    \(\(y\  = {{y00, y01, y02, y03}, {y10, y11, y12, 
            y13}};\)\), "\[IndentingNewLine]", 
    \(\(X\  = {X0, X1, X2, X3, X4};\)\), "\[IndentingNewLine]", 
    \(\(z = {z1};\)\), "\[IndentingNewLine]", 
    \(\(vars\  = Join[X, z];\)\), "\[IndentingNewLine]", 
    \(\(z = Prepend[z, 1];\)\)}], "Input",
  CellLabel->"In[106]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["set the true coefficients and offsets", "Text",
  FontSize->18,
  FontWeight->"Plain",
  FontVariations->{"CompatibilityType"->0}]], "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Xtrue\  = \ Table[0, {i, LL}];\)\), "\[IndentingNewLine]", 
    \(\(Xtrue[\([KK + 1]\)] = 
        Random[Integer, {\(-10\), 10}];\)\), "\[IndentingNewLine]", 
    \(For[i = 1, 
      i < KK + 1, \(i++\), {Xtrue[\([KK + 1 + i]\)] = 
          Random[Integer, {\(-10\), 10}] + 
            I*Random[
                Integer, {\(-10\), 10}]; \[IndentingNewLine]Xtrue[\([KK + 1 - 
                i]\)] = 
          Conjugate[
            Xtrue[\([KK + 1 + 
                  i]\)]];\[IndentingNewLine]}]\), "\[IndentingNewLine]", 
    \(\(ttrue\  = \ Table[0, {i, MM}];\)\), "\[IndentingNewLine]", 
    \(\(For[i = 2, i < MM + 1, \(i++\), 
        ttrue[\([i]\)] = 
          Rational[Random[Integer, {1, 10}], 
            11]];\)\), "\[IndentingNewLine]", 
    \(ztrue = Exp[I*2*\[Pi]*ttrue]\), "\[IndentingNewLine]", 
    \({Xtrue, ttrue, ztrue}\), "\[IndentingNewLine]", 
    \(N[ztrue[\([2]\)]]\)}], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \({1, \[ExponentialE]\^\(-\(\(10\ \[ImaginaryI]\ \[Pi]\)\/11\)\)}\)], \
"Output",
  CellLabel->"Out[116]="],

Cell[BoxData[
    \({{5 - \[ImaginaryI], \(-3\)\ \[ImaginaryI], \(-6\), 3\ \[ImaginaryI], 
        5 + \[ImaginaryI]}, {0, 
        6\/11}, {1, \[ExponentialE]\^\(-\(\(10\ \[ImaginaryI]\ \
\[Pi]\)\/11\)\)}}\)], "Output",
  CellLabel->"Out[117]="],

Cell[BoxData[
    \(\(-0.9594929736144973`\) - 
      0.28173255684143006`\ \[ImaginaryI]\)], "Output",
  CellLabel->"Out[118]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["set polynomials", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(Plist = Table[0, {i, MM*NN}]; 
    Plistmatrix = 
      Table[0, {i, MM*NN}, {j, LL + 1}];\), "\[IndentingNewLine]", 
    \(\(For[n = 1, 
        n < NN + 1, \(n++\), \[IndentingNewLine]\(For[m = 1, 
            m < MM + 
                1, \(m++\), {\[IndentingNewLine]Plist[\([n + 
                      NN*\((m - 1)\)]\)] = \(-\(y[\([m]\)]\)[\([n]\)]\) 
                  z[\([m]\)]^KK; 
              Plistmatrix[\([n + NN*\((m - 1)\), 
                    LL + 1]\)] = \(y[\([m]\)]\)[\([n]\)] 
                  z[\([m]\)]^KK; \[IndentingNewLine]For[l = 1, 
                l < LL + 
                    1, \(l++\), {\[IndentingNewLine]Plist[\([n + 
                          NN*\((m - 1)\)]\)] = 
                    Plist[\([n + NN*\((m - 1)\)]\)] + 
                      Exp[I*2*\[Pi]*\((n - 1)\) \((l - KK - 1)\)/NN] 
                        z[\([m]\)]^\((l - 1)\)*
                        X[\([l]\)]; \[IndentingNewLine]Plistmatrix[\([n + 
                          NN*\((m - 1)\), l]\)] = 
                    Exp[I*2*\[Pi]*\((n - 1)\) \((l - KK - 1)\)/NN] 
                      z[\([m]\)]^\((l - 
                            1)\);\[IndentingNewLine]}];\[IndentingNewLine]}\
\[IndentingNewLine]];\)\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(\(Plist;\)\), "\[IndentingNewLine]", 
    \(Plistmatrix\)}], "Input",
  CellLabel->"In[119]:="],

Cell[BoxData[
    \({{1, 1, 1, 1, 1, y00}, {\(-1\), \(-\[ImaginaryI]\), 
        1, \[ImaginaryI], \(-1\), y01}, {1, \(-1\), 1, \(-1\), 1, 
        y02}, {\(-1\), \[ImaginaryI], 1, \(-\[ImaginaryI]\), \(-1\), 
        y03}, {1, z1, z1\^2, z1\^3, z1\^4, 
        y10\ z1\^2}, {\(-1\), \(-\[ImaginaryI]\)\ z1, 
        z1\^2, \[ImaginaryI]\ z1\^3, \(-z1\^4\), y11\ z1\^2}, {1, \(-z1\), 
        z1\^2, \(-z1\^3\), z1\^4, y12\ z1\^2}, {\(-1\), \[ImaginaryI]\ z1, 
        z1\^2, \(-\[ImaginaryI]\)\ z1\^3, \(-z1\^4\), 
        y13\ z1\^2}}\)], "Output",
  CellLabel->"Out[122]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
compute sample values (sample values are supposed to be real)\
\>", \
"Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(reprules = Table[0, {i, LL + MM - 1}];\)\), "\[IndentingNewLine]", 
    \(\(For[l = 1, l < LL + 1, \(l++\), 
        reprules[\([l]\)] = 
          X[\([l]\)] \[Rule] Xtrue[\([l]\)]];\)\), "\[IndentingNewLine]", 
    \(\(For[m = 1, m < MM, \(m++\), 
        reprules[\([LL + m]\)] = 
          z[\([m + 1]\)] \[Rule] 
            ztrue[\([m + 1]\)]];\)\), "\[IndentingNewLine]", 
    \(test := Plist\  /. reprules\), "\[IndentingNewLine]", 
    \(samples = Simplify[Solve[test \[Equal] 0]]\), "\[IndentingNewLine]", 
    \(realsamples = Re[y /. N[samples[\([1]\)]]]\)}], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \({{y00 \[Rule] 4, y01 \[Rule] \(-22\), y02 \[Rule] 4, 
        y03 \[Rule] \(-10\), 
        y10 \[Rule] \(-6\) + \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\^\(2/11\) \
+ 3\ \((\(-1\))\)\^\(9/22\) - 
            3\ \((\(-1\))\)\^\(13/22\) - \((5 - \[ImaginaryI])\)\ \
\((\(-1\))\)\^\(9/11\), 
        y11 \[Rule] \(-6\) + 
            3\ \((\(-1\))\)\^\(1/11\) - \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\
\^\(2/11\) + \((5 - \[ImaginaryI])\)\ \((\(-1\))\)\^\(9/11\) - 
            3\ \((\(-1\))\)\^\(10/11\), 
        y12 \[Rule] \(-6\) + \((5 + \[ImaginaryI])\)\ \((\(-1\))\)\^\(2/11\) \
- 3\ \((\(-1\))\)\^\(9/22\) + 
            3\ \((\(-1\))\)\^\(13/22\) - \((5 - \[ImaginaryI])\)\ \
\((\(-1\))\)\^\(9/11\), 
        y13 \[Rule] \(-6\) - 
            3\ \((\(-1\))\)\^\(1/11\) + \((1 + 
                  5\ \[ImaginaryI])\)\ \((\(-1\))\)\^\(7/22\) - \((1 - 
                  5\ \[ImaginaryI])\)\ \((\(-1\))\)\^\(15/22\) + 
            3\ \((\(-1\))\)\^\(10/11\)}}\)], "Output",
  CellLabel->"Out[127]="],

Cell[BoxData[
    \({{4.`, \(-22.`\), 
        4.`, \(-10.`\)}, {3.021649034449195`, \(-7.574295851713632`\), \
\(-0.359141647647959`\), \(-19.088211535087595`\)}}\)], "Output",
  CellLabel->"Out[128]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["add Gaussian noise to the samples", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(<< Statistics`ContinuousDistributions`\), "\[IndentingNewLine]", 
    \(\(noise = 
        Table[Random[NormalDistribution[0, 1]], {i, MM}, {j, 
            NN}];\)\), "\[IndentingNewLine]", 
    \(noisysamples = N[realsamples + noise]\)}], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \({{3.4844899221494563`, \(-21.2467933966749`\), 
        3.667159566365723`, \(-9.531024276259005`\)}, {2.091677416681718`, \
\(-7.447950556525134`\), 
        0.729979434423164`, \(-19.30783212638691`\)}}\)], "Output",
  CellLabel->"Out[131]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert sample values into replacement rules", "Section"],

Cell[BoxData[{
    \(\(newsamples = Table[0, {i, MM}, {j, NN}];\)\), "\[IndentingNewLine]", 
    \(\(noiselesssamples = 
        Table[0, {i, MM}, {j, NN}];\)\), "\[IndentingNewLine]", 
    \(For[m = 1, 
      m < MM + 1, \(m++\), \[IndentingNewLine]For[n = 1, 
        n < NN + 
            1, \(n++\), {\[IndentingNewLine]noiselesssamples[\([m, n]\)] = 
            y[\([m, n]\)] \[Rule] realsamples[\([m, n]\)]; 
          newsamples[\([m, n]\)] = 
            y[\([m, n]\)] \[Rule] 
              noisysamples[\([m, 
                  n]\)];\[IndentingNewLine]}]]\), "\[IndentingNewLine]", 
    \(\(noiselesssamples = 
        Flatten[noiselesssamples];\)\), "\[IndentingNewLine]", 
    \(\(samples = Flatten[newsamples];\)\)}], "Input",
  CellLabel->"In[132]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["compute polynomials", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[BoxData[{
    \(\(polylist\  = Plist /. samples;\)\), "\[IndentingNewLine]", 
    \(\(polylistnoiseless = 
        Plist /. noiselesssamples;\)\), "\[IndentingNewLine]", 
    \(\(polylistmatrix = Plistmatrix /. {samples};\)\)}], "Input",
  CellLabel->"In[137]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["compute grobner basis", "Section",
  FontVariations->{"CompatibilityType"->0}],

Cell[CellGroupData[{

Cell["\<\
first method: direct computation of a Grobner basis for all \
polynomials\
\>", "Subsubsection"],

Cell["gb = GroebnerBasis[polylist, vars]", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
second method: eliminate X using RowReduce (on a matrix); uses \
Gaussian elimination\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Testmatrix = 
        RowReduce[polylistmatrix[\([1, Range[LL]]\)]];\)\), "\n", 
    \(Expand[Testmatrix]\), "\[IndentingNewLine]", 
    \(\(Xsolved = Table[0, {i, LL}];\)\), "\n", 
    \(\(For[i = 1, 
        i < LL + 1, \(i++\), \(Xsolved[\([i]\)] = 
            X[\([i]\)] \[Rule] \ 
              Testmatrix[\([i, LL + 1]\)];\)];\)\)}], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \({{1, 0, 0, 0, 
        0, \((\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\) + \(\(\(9.48236679036227`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI]\)\/\(\(-1\) + z1\^4\) - \
\(\((\(\(0.04566741105406669`\)\(\[InvisibleSpace]\)\) + 2.928942280103973`\ \
\[ImaginaryI])\)\ z1\)\/\(\(-1\) + z1\^4\) - \(\((\(\(7.998219462786398`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI])\)\ z1\^2\)\/\(\(-1\) + z1\^4\) - \
\(\((\(\(0.04566741105406669`\)\(\[InvisibleSpace]\)\) - 2.928942280103973`\ \
\[ImaginaryI])\)\ z1\^3\)\/\(\(-1\) + z1\^4\)}, {0, 1, 0, 0, 
        0, \(-0.04566741105406669`\) - 
          2.928942280103973`\ \[ImaginaryI]}, {0, 0, 1, 0, 
        0, \(-5.906542046104681`\) + 0.`\ \[ImaginaryI]}, {0, 0, 0, 1, 
        0, \(-0.04566741105406669`\) + 
          2.928942280103973`\ \[ImaginaryI]}, {0, 0, 0, 0, 
        1, \(-\(\(\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 
                  0.`\ \[ImaginaryI]\)\/\(\(-1\) + 
                  z1\^4\)\)\) + \(\((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) + 2.928942280103973`\ \[ImaginaryI])\)\ z1\)\/\(\(-1\) \
+ z1\^4\) + \(\((\(\(7.998219462786398`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI])\)\ z1\^2\)\/\(\(-1\) + z1\^4\) + \(\((\(\(0.04566741105406669`\
\)\(\[InvisibleSpace]\)\) - 2.928942280103973`\ \[ImaginaryI])\)\ z1\^3\)\/\(\
\(-1\) + z1\^4\)}}\)], "Output",
  CellLabel->"Out[141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(tlist = polylist[\([Range[LL + 1, MM*NN]]\)] /. Xsolved\)], "Input",
  CellLabel->"In[144]:="],

Cell[BoxData[
    \({\((\(-9.48236679036227`\) + 
            0.`\ \[ImaginaryI])\) - \((\(\(2.928942280103973`\)\(\
\[InvisibleSpace]\)\) - 
              0.04566741105406669`\ \[ImaginaryI])\)\ z1 + \
\((\(\(1.5414085104204531`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^2 - \((\(\(2.928942280103973`\)\(\
\[InvisibleSpace]\)\) + 
              0.04566741105406669`\ \[ImaginaryI])\)\ z1\^3 - \
\(\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI]\)\/\(\(-1\) + z1\^4\) + \(\((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) + 2.928942280103973`\ \[ImaginaryI])\)\ z1\)\/\(\(-1\) \
+ z1\^4\) + \(\((\(\(7.998219462786398`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI])\)\ z1\^2\)\/\(\(-1\) + z1\^4\) + \(\((\(\(0.04566741105406669`\
\)\(\[InvisibleSpace]\)\) - 2.928942280103973`\ \[ImaginaryI])\)\ z1\^3\)\/\(\
\(-1\) + z1\^4\) - \(\(1\/\(256\ \((\(-1\) + 
                  z1\^4)\)\)\)\((\[ImaginaryI]\ z1\^4\ \((\(-32\)\ \
\[ImaginaryI]\ \((\((\(\(70.68387460966923`\)\(\[InvisibleSpace]\)\) + 
                            0.3653392884325335`\ \[ImaginaryI])\)\ \((\(-1\) \
+ z1\^2)\) - 4\ \((\(-0.18266964421626675`\)\ \((\(-1\) + z1)\) - 
                            2\ \((\(-3.4844899221494563`\) + 
                                  2.091677416681718`\ z1\^2)\))\))\) - \
\((\(\(93.72615296332714`\)\(\[InvisibleSpace]\)\) + 
                      1.461357153730134`\ \[ImaginaryI])\)\ \((\(-8\)\ \
\[ImaginaryI]\ \((\(-1\) + z1\^2)\) - 
                      4\ \((2\ \((\(-1\) + z1)\) - 
                            2\ \((\(-1\) + 
                                  z1\^3)\))\))\))\))\)\), \
\((\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 
            0.`\ \[ImaginaryI])\) + \((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) + 
              2.928942280103973`\ \[ImaginaryI])\)\ z1 - \
\((\(\(6.636521480527845`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^2 + \((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) - 
              2.928942280103973`\ \[ImaginaryI])\)\ z1\^3 + \
\(\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI]\)\/\(\(-1\) + z1\^4\) - \(\((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) + 2.928942280103973`\ \[ImaginaryI])\)\ z1\)\/\(\(-1\) \
+ z1\^4\) - \(\((\(\(7.998219462786398`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI])\)\ z1\^2\)\/\(\(-1\) + z1\^4\) - \(\((\(\(0.04566741105406669`\
\)\(\[InvisibleSpace]\)\) - 2.928942280103973`\ \[ImaginaryI])\)\ z1\^3\)\/\(\
\(-1\) + z1\^4\) + \(\(1\/\(256\ \((\(-1\) + 
                  z1\^4)\)\)\)\((\[ImaginaryI]\ z1\^4\ \((\(-32\)\ \
\[ImaginaryI]\ \((\((\(\(70.68387460966923`\)\(\[InvisibleSpace]\)\) + 
                            0.3653392884325335`\ \[ImaginaryI])\)\ \((\(-1\) \
+ z1\^2)\) - 4\ \((\(-0.18266964421626675`\)\ \((\(-1\) + z1)\) - 
                            2\ \((\(-3.4844899221494563`\) + 
                                  2.091677416681718`\ z1\^2)\))\))\) - \
\((\(\(93.72615296332714`\)\(\[InvisibleSpace]\)\) + 
                      1.461357153730134`\ \[ImaginaryI])\)\ \((\(-8\)\ \
\[ImaginaryI]\ \((\(-1\) + z1\^2)\) - 
                      4\ \((2\ \((\(-1\) + z1)\) - 
                            2\ \((\(-1\) + 
                                  z1\^3)\))\))\))\))\)\), \
\((\(-9.48236679036227`\) + 
            0.`\ \[ImaginaryI])\) + \((\(\(2.928942280103973`\)\(\
\[InvisibleSpace]\)\) - 
              0.04566741105406669`\ \[ImaginaryI])\)\ z1 + \
\((\(\(13.401290080282228`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^2 + \((\(\(2.928942280103973`\)\(\
\[InvisibleSpace]\)\) + 
              0.04566741105406669`\ \[ImaginaryI])\)\ z1\^3 - \
\(\(\(9.48236679036227`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI]\)\/\(\(-1\) + z1\^4\) + \(\((\(\(0.04566741105406669`\)\(\
\[InvisibleSpace]\)\) + 2.928942280103973`\ \[ImaginaryI])\)\ z1\)\/\(\(-1\) \
+ z1\^4\) + \(\((\(\(7.998219462786398`\)\(\[InvisibleSpace]\)\) + 0.`\ \
\[ImaginaryI])\)\ z1\^2\)\/\(\(-1\) + z1\^4\) + \(\((\(\(0.04566741105406669`\
\)\(\[InvisibleSpace]\)\) - 2.928942280103973`\ \[ImaginaryI])\)\ z1\^3\)\/\(\
\(-1\) + z1\^4\) - \(\(1\/\(256\ \((\(-1\) + 
                  z1\^4)\)\)\)\((\[ImaginaryI]\ z1\^4\ \((\(-32\)\ \
\[ImaginaryI]\ \((\((\(\(70.68387460966923`\)\(\[InvisibleSpace]\)\) + 
                            0.3653392884325335`\ \[ImaginaryI])\)\ \((\(-1\) \
+ z1\^2)\) - 4\ \((\(-0.18266964421626675`\)\ \((\(-1\) + z1)\) - 
                            2\ \((\(-3.4844899221494563`\) + 
                                  2.091677416681718`\ z1\^2)\))\))\) - \
\((\(\(93.72615296332714`\)\(\[InvisibleSpace]\)\) + 
                      1.461357153730134`\ \[ImaginaryI])\)\ \((\(-8\)\ \
\[ImaginaryI]\ \((\(-1\) + z1\^2)\) - 
                      4\ \((2\ \((\(-1\) + z1)\) - 
                            2\ \((\(-1\) + 
                                  z1\^3)\))\))\))\))\)\)}\)], "Output",
  CellLabel->"Out[144]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["multiply list of polynomials by their common denominator", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(newtlist = Table[0, {i, Length[tlist]}]; 
    For[i = 1, 
      i < Length[tlist] + 1, \(i++\), \[IndentingNewLine]newtlist[\([i]\)] = 
        Simplify[
          Denominator[Together[tlist[\([i]\)]]]*
            tlist[\([i]\)]]]\), "\[IndentingNewLine]", 
    \(\(tlist = newtlist;\)\), "\[IndentingNewLine]", 
    \(Expand[tlist]\)}], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \({\((\(\(2.9746096911580397`\)\(\[InvisibleSpace]\)\) + 
              2.8832748690499064`\ \[ImaginaryI])\)\ z1 + \
\((\(\(6.456810952365945`\)\(\[InvisibleSpace]\)\) + 
              8.881784197001252`*^-16\ \[ImaginaryI])\)\ z1\^2 + \
\((\(\(2.9746096911580397`\)\(\[InvisibleSpace]\)\) - 
              2.8832748690499064`\ \[ImaginaryI])\)\ z1\^3 + \((\(\(0.`\)\(\
\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^4 - \((\(\(2.9746096911580393`\)\(\
\[InvisibleSpace]\)\) + 
              2.883274869049906`\ \[ImaginaryI])\)\ z1\^5 - \
\((\(\(6.456810952365945`\)\(\[InvisibleSpace]\)\) + 
              4.440892098500626`*^-16\ \[ImaginaryI])\)\ z1\^6 - \
\((\(\(2.9746096911580397`\)\(\[InvisibleSpace]\)\) - 
              2.8832748690499064`\ \[ImaginaryI])\)\ z1\^7, \
\((\(-0.09133482210813348`\) - 
              5.857884560207946`\ \[ImaginaryI])\)\ z1 - \
\((\(\(1.361697982258553`\)\(\[InvisibleSpace]\)\) + 
              8.881784197001252`*^-16\ \[ImaginaryI])\)\ z1\^2 - \
\((\(\(0.09133482210813337`\)\(\[InvisibleSpace]\)\) - 
              5.857884560207945`\ \[ImaginaryI])\)\ z1\^3 + \((\(\(0.`\)\(\
\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^4 + \((\(\(0.09133482210813337`\)\(\
\[InvisibleSpace]\)\) + 
              5.857884560207945`\ \[ImaginaryI])\)\ z1\^5 + \
\((\(\(1.361697982258553`\)\(\[InvisibleSpace]\)\) - 
              8.881784197001252`*^-16\ \[ImaginaryI])\)\ z1\^6 + \
\((\(\(0.09133482210813337`\)\(\[InvisibleSpace]\)\) - 
              5.857884560207946`\ \[ImaginaryI])\)\ z1\^7, \
\((\(-2.8832748690499073`\) + 
              2.9746096911580397`\ \[ImaginaryI])\)\ z1 - \
\((\(\(5.40307061749583`\)\(\[InvisibleSpace]\)\) - 
              4.440892098500626`*^-16\ \[ImaginaryI])\)\ z1\^2 - \
\((\(\(2.8832748690499077`\)\(\[InvisibleSpace]\)\) + 
              2.97460969115804`\ \[ImaginaryI])\)\ z1\^3 - \
\((1.7763568394002505`*^-15 + 
              8.881784197001252`*^-16\ \[ImaginaryI])\)\ z1\^4 + \
\((\(\(2.883274869049906`\)\(\[InvisibleSpace]\)\) - 
              2.9746096911580397`\ \[ImaginaryI])\)\ z1\^5 + \
\((\(\(5.40307061749583`\)\(\[InvisibleSpace]\)\) + 
              0.`\ \[ImaginaryI])\)\ z1\^6 + \((\(\(2.8832748690499064`\)\(\
\[InvisibleSpace]\)\) + 
              2.9746096911580397`\ \[ImaginaryI])\)\ z1\^7}\)], "Output",
  CellLabel->"Out[147]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
compute Grobner bases for the offsets for all critical sets of \
polynomials\
\>", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(coeff = Range[1, Length[tlist]];\)\), "\[IndentingNewLine]", 
    \(\(coefflists = Subsets[coeff, {MM - 1}];\)\), "\[IndentingNewLine]", 
    \(\(GroebnerShifts = 
        Table[0, {i, Length[tlist]}];\)\), "\[IndentingNewLine]", 
    \(For[i = 1, i < Length[coefflists] + 1, \(i++\), 
      GroebnerShifts[\([i]\)] = 
        GroebnerBasis[tlist[\([coefflists[\([i]\)]]\)], 
          z[\([Range[2, MM]]\)]]]\), "\[IndentingNewLine]", 
    \(GroebnerShifts\)}], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \({{\((\(-0.031175972402324087`\) - 
                0.9995139112312391`\ \[ImaginaryI])\)\ z1 - \
\((\(\(1.1191566295596704`\)\(\[InvisibleSpace]\)\) + 
                1.0847931391239973`\ \[ImaginaryI])\)\ z1\^2 - \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 
                0.`\ \[ImaginaryI])\)\ z1\^3 + \((\(\(0.031175972402324087`\)\
\(\[InvisibleSpace]\)\) + 
                0.999513911231239`\ \[ImaginaryI])\)\ z1\^5 + \
\((\(\(1.1191566295596707`\)\(\[InvisibleSpace]\)\) + 
                1.084793139123997`\ \[ImaginaryI])\)\ z1\^6 + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 
                0.`\ \[ImaginaryI])\)\ z1\^7}, \
{\((\(\(0.9995139112312391`\)\(\[InvisibleSpace]\)\) - 
                0.03117597240232409`\ \[ImaginaryI])\)\ z1 - \
\((\(\(0.0036235144512376883`\)\(\[InvisibleSpace]\)\) + 
                0.23239908796741746`\ \[ImaginaryI])\)\ z1\^2 - \
\((\(\(0.9999999999999999`\)\(\[InvisibleSpace]\)\) + 
                2.3685026133269558`*^-18\ \[ImaginaryI])\)\ z1\^3 - \
\((\(\(0.999513911231239`\)\(\[InvisibleSpace]\)\) - 
                0.031175972402324073`\ \[ImaginaryI])\)\ z1\^5 + \
\((\(\(0.0036235144512379914`\)\(\[InvisibleSpace]\)\) + 
                0.23239908796741746`\ \[ImaginaryI])\)\ z1\^6 + \
\((\(\(1.`\)\(\[InvisibleSpace]\)\) + 
                0.`\ \[ImaginaryI])\)\ z1\^7}, \
{\((\(\(0.031175972402323934`\)\(\[InvisibleSpace]\)\) + 
                0.9995139112312393`\ \[ImaginaryI])\)\ z1 - \
\((\(\(0.9077567826132857`\)\(\[InvisibleSpace]\)\) - 
                0.9365122110836752`\ \[ImaginaryI])\)\ z1\^2 - \
\((\(\(1.0000000000000002`\)\(\[InvisibleSpace]\)\) - 
                1.539476335875987`*^-16\ \[ImaginaryI])\)\ z1\^3 - \
\((\(\(0.031175972402324163`\)\(\[InvisibleSpace]\)\) + 
                0.999513911231239`\ \[ImaginaryI])\)\ z1\^5 + \
\((\(\(0.9077567826132857`\)\(\[InvisibleSpace]\)\) - 
                0.9365122110836752`\ \[ImaginaryI])\)\ z1\^6 + \((\(\(1.`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI])\)\ z1\^7}}\)], "Output",
  CellLabel->"Out[152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Tsolved = 
        Table[0, {i, Length[GroebnerShifts]}];\)\), "\[IndentingNewLine]", 
    \(For\ [i = 1, 
      i < Length[GroebnerShifts] + 
          1, \(i++\), {\(Tsolved[\([i]\)] = 
            Solve[GroebnerShifts[\([i]\)] \[Equal] 0, 
              z[\([Range[2, 
                    MM]]\)]];\)\[IndentingNewLine]}]\), \
"\[IndentingNewLine]", 
    \(Tsolved = N[Tsolved]\)}], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \({{{z1 \[Rule] \(-1.0000000000000002`\)}, {z1 \[Rule] \
\(-0.9957181720804651`\) - 
              0.09244090971391783`\ \[ImaginaryI]}, {z1 \[Rule] \
\(-0.12343845747920558`\) - 
              0.9923522294100788`\ \[ImaginaryI]}, {z1 \[Rule] \
\(-1.4262387914355031`*^-18\) + 1.`\ \[ImaginaryI]}, {z1 \[Rule] 
            0.`}, {z1 \[Rule] 
            3.991378963931102`*^-16 - 
              1.0000000000000004`\ \[ImaginaryI]}, {z1 \[Rule] 
            1.`}}, {{z1 \[Rule] \(-1.0000000000000002`\) - 
              4.121702978920894`*^-15\ \[ImaginaryI]}, {z1 \[Rule] \
\(-0.9949152873742118`\) - 
              0.10071529650002563`\ \[ImaginaryI]}, {z1 \[Rule] \
\(-9.36092289690938`*^-17\) - 
              1.`\ \[ImaginaryI]}, {z1 \[Rule] \(-8.283630176078355`*^-17\) + 
              0.9999999999999999`\ \[ImaginaryI]}, {z1 \[Rule] 
            0.`}, {z1 \[Rule] \(\(0.991291772922974`\)\(\[InvisibleSpace]\)\) \
- 0.1316837914673876`\ \[ImaginaryI]}, {z1 \[Rule] 
            1.`}}, {{z1 \[Rule] \(-0.9999999999999982`\)}, {z1 \[Rule] \
\(-0.9982350811929301`\) - 
              0.059386216210058795`\ \[ImaginaryI]}, {z1 \[Rule] \
\(-9.839058660530022`*^-19\) - 
              0.9999999999999999`\ \[ImaginaryI]}, {z1 \[Rule] 
            0.`}, {z1 \[Rule] 
            1.0656780792631762`*^-15 + 
              1.0000000000000002`\ \[ImaginaryI]}, {z1 \[Rule] \
\(\(0.09047829857964144`\)\(\[InvisibleSpace]\)\) + 
              0.9958984272937338`\ \[ImaginaryI]}, {z1 \[Rule] 
            1.`}}}\)], "Output",
  CellLabel->"Out[155]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["eliminate shifts that do not give a solution", \
"Subsubsection",
  FontSize->16]], "Section",
  FontSize->18],

Cell["\<\
- shifts not on the unit circle
- offset vectors that are all 1 (or -1)
- offset vectors that do not give a solution to the equations from \
polylistmatrix\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(zsolved = Table[{}, {i, Length[GroebnerShifts]}]; 
    For[i = 1, 
      i < Length[GroebnerShifts] + 1, \(i++\), {\[IndentingNewLine]Temp = 
          z /. Tsolved[\([i]\)]; \[IndentingNewLine]zabs = 
          Abs[Temp]; \[IndentingNewLine]onecount = 
          Table[0, {i, Length[Tsolved[\([i]\)]]}]; \[IndentingNewLine]For[
          k = 1, k < 
            Length[Tsolved[\([i]\)]] + 1, \(k++\), {\[IndentingNewLine]ztmp = 
              1; \[IndentingNewLine]onecount[\([k]\)] = 
              0; \[IndentingNewLine]For[m = 1, 
              m < MM + 1, \(m++\), {\[IndentingNewLine]If[
                  zabs[\([k, m]\)] > 1 + 10\^\(-5\) || 
                    zabs[\([k, m]\)] < 1 - 10\^\(-5\), 
                  ztmp = 0]; \[IndentingNewLine]If[
                  Abs[Re[Temp[\([k, m]\)]]] < 1 + 10\^\(-5\) && 
                    Abs[Re[Temp[\([k, m]\)]]] > 1 - 10\^\(-5\), 
                  onecount[\([k]\)] = 
                    onecount[\([k]\)] + 
                      1];\[IndentingNewLine]}]; \[IndentingNewLine]If[
              ztmp \[Equal] 1 && 
                onecount[\([k]\)] \[NotEqual] MM, \[IndentingNewLine]{tmp1 = 
                  polylistmatrix /. Tsolved[\([i, k]\)]; 
                tmp2 = \(X /. {Xsolved}\) /. 
                    Tsolved[\([i, k]\)]; \[IndentingNewLine]tmp2 = 
                  Append[tmp2[\([1]\)], \(-1\)]; \[IndentingNewLine]If[
                  Norm[tmp1 . tmp2] < 10, 
                  zsolved[\([i]\)] = 
                    Append[zsolved[\([i]\)], 
                      Tsolved[\([i, 
                          k]\)]]]}];\[IndentingNewLine]}]\[IndentingNewLine]}]\
\), "\[IndentingNewLine]", 
    \(zsolved\)}], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \({{{z1 \[Rule] \(-0.9957181720804651`\) - 
              0.09244090971391783`\ \[ImaginaryI]}}, {{z1 \[Rule] \
\(-0.9949152873742118`\) - 
              0.10071529650002563`\ \[ImaginaryI]}}, {{z1 \[Rule] \
\(-0.9982350811929301`\) - 
              0.059386216210058795`\ \[ImaginaryI]}}}\)], "Output",
  CellLabel->"Out[157]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["compute the average of the solutions", \
"Subsubsection",
  FontSize->16]], "Section",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(tSolved = 
        Re[Log[\((z /. zsolved)\)]/\((2  \[Pi]*
                I)\)];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Offsets  t:\>", 
        NN*\((tSolved + 1)\)];\)\), "\[IndentingNewLine]", 
    \(\(tSolvedAvg = Flatten[Mean[tSolved]];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Average offset t:\>", 
        NN*\((tSolvedAvg + 1)\)];\)\), "\[IndentingNewLine]", 
    \(\(zSolvedAvgRule = Table[0, {i, MM - 1}];\)\), "\[IndentingNewLine]", 
    \(\(For[i = 2, i < MM + 1, \(i++\), 
        zSolvedAvgRule[\([i - 1]\)] = 
          z[\([i]\)] \[Rule] 
            Exp[I*2*\[Pi]*tSolvedAvg[\([i]\)]]];\)\), "\[IndentingNewLine]", 
    \(zSolvedAvgRule\)}], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    InterpretationBox[\("Offsets  t:"\[InvisibleSpace]{{{4, 
              2.0589338498028944`}}, {{4, 2.064226243407326`}}, {{4, 
              2.037828696924605`}}}\),
      SequenceForm[ 
      "Offsets  t:", {{{4, 2.0589338498028944}}, {{4, 2.064226243407326}}, {{
        4, 2.0378286969246049}}}],
      Editable->False]], "Print",
  CellLabel->"From In[158]:="],

Cell[BoxData[
    InterpretationBox[\("Average offset t:"\[InvisibleSpace]{4, 
          2.0536629300449416`}\),
      SequenceForm[ "Average offset t:", {4, 2.0536629300449416}],
      Editable->False]], "Print",
  CellLabel->"From In[158]:="],

Cell[BoxData[
    \({z1 \[Rule] \(-0.9964494032280782`\) - 
          0.08419374564899008`\ \[ImaginaryI]}\)], "Output",
  CellLabel->"Out[164]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Xcomputed = 
        Table[0, {i, Length[zSolvedAvgRule]}];\)\), "\[IndentingNewLine]", 
    \(For[i = 1, 
      i < Length[zSolvedAvgRule] + 
          1, \(i++\), {\[IndentingNewLine]\(Xcomputed[\([i]\)] = 
            Xsolved /. 
              zSolvedAvgRule;\)\[IndentingNewLine]}]\), \
"\[IndentingNewLine]", 
    \(Xcomputed = Flatten[Xcomputed]\)}], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \({X0 \[Rule] \(\(4.741183395181137`\)\(\[InvisibleSpace]\)\) - 
          5.762936927109519`\ \[ImaginaryI], 
      X1 \[Rule] \(-0.04566741105406669`\) - 
          2.928942280103973`\ \[ImaginaryI], 
      X2 \[Rule] \(-5.906542046104681`\) + 0.`\ \[ImaginaryI], 
      X3 \[Rule] \(-0.04566741105406669`\) + 
          2.928942280103973`\ \[ImaginaryI], 
      X4 \[Rule] \(\(4.741183395181135`\)\(\[InvisibleSpace]\)\) + 
          5.7629369271095205`\ \[ImaginaryI]}\)], "Output",
  CellLabel->"Out[167]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compute the offsets t from the z, and display solutions with the \
norm of the error\
\>", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(TfromZ = 
        N[Log[z /. 
                zSolvedAvgRule]/\((I*2*\[Pi])\)];\)\), "\[IndentingNewLine]", \

    \(\(\(For[j = 1, j < MM + 1, \(j++\), 
      If[Im[TfromZ[\([j]\)]] < 
          10\^\(-10\), \(TfromZ[\([j]\)] = 
            Re[TfromZ[\([j]\)]];\)]]\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(CoeffMatrixExactOffsets = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {1, LL}] /. 
          Take[reprules, \(-\((MM - 1)\)\)];\)\), "\[IndentingNewLine]", 
    \(\(SampleVectorExactOffsets = 
        Take[polylistmatrix[\([1]\)], {1, MM*NN}, {LL + 1, LL + 1}] /. 
          Take[reprules, \(-\((MM - 1)\)\)];\)\), "\[IndentingNewLine]", 
    \(\(\(XcomputedExactOffsets = 
        PseudoInverse[CoeffMatrixExactOffsets] . 
          SampleVectorExactOffsets;\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(Print["\<Solution: \>"];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Exact offsets:\>", 
        N[ttrue], "\<\n\>", "\<Estimated offsets:\>", 
        tSolvedAvg];\)\), "\[IndentingNewLine]", 
    \(\(Print["\<Exact coefficients:\>", 
        Xtrue, "\<\n\n\>", "\<Estimated coefficients with exact offsets:\>", 
        Flatten[XcomputedExactOffsets], "\<\n\>", "\<Coefficient error using \
exact shifts: \>", 
        Norm[XcomputedExactOffsets - Xtrue]/
          Norm[Xtrue], "\<\n\n\>", "\<Estimated coefficients with estimated \
shifts:\>", 
        Xcomputed, "\<\n\>", "\<Coefficient error using estimated shifts: \
\>", Norm[\((X /. Xcomputed)\) - Xtrue]/
          Norm[Xtrue]];\)\), "\[IndentingNewLine]", 
    \(\(Print[
        Norm[\(polylistnoiseless /. {Xcomputed}\) /. 
            zSolvedAvgRule]];\)\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \("Solution: "\)], "Print",
  CellLabel->"From In[168]:="],

Cell[BoxData[
    InterpretationBox[\("Exact offsets:"\[InvisibleSpace]{0.`, 
          0.5454545454545454`}\[InvisibleSpace]"\n\
"\[InvisibleSpace]"Estimated offsets:"\[InvisibleSpace]{0, \
\(-0.4865842674887646`\)}\),
      SequenceForm[ 
      "Exact offsets:", {0.0, 0.54545454545454541}, "\n", 
        "Estimated offsets:", {0, -0.4865842674887646}],
      Editable->False]], "Print",
  CellLabel->"From In[168]:="],

Cell[BoxData[
    InterpretationBox[\("Exact coefficients:"\[InvisibleSpace]{5 - \
\[ImaginaryI], \(-3\)\ \[ImaginaryI], \(-6\), 3\ \[ImaginaryI], 
          5 + \[ImaginaryI]}\[InvisibleSpace]"\n\n\
"\[InvisibleSpace]"Estimated coefficients with exact offsets:"\
\[InvisibleSpace]{\(\(4.741183395181135`\)\(\[InvisibleSpace]\)\) - 
            0.5389111029680582`\ \[ImaginaryI], \(\(0.23151318351691413`\)\(\
\[InvisibleSpace]\)\) - 
            2.934859601098233`\ \[ImaginaryI], \(-5.945036752028237`\) - 
            1.0375309099882165`*^-16\ \[ImaginaryI], \(\(0.23151318351691505`\
\)\(\[InvisibleSpace]\)\) + 
            2.9348596010982324`\ \[ImaginaryI], \(\(4.741183395181135`\)\(\
\[InvisibleSpace]\)\) + 
            0.5389111029680582`\ \[ImaginaryI]}\[InvisibleSpace]"\n"\
\[InvisibleSpace]"Coefficient error using exact shifts: \
"\[InvisibleSpace]0.07996946480000004`\[InvisibleSpace]"\n\n\
"\[InvisibleSpace]"Estimated coefficients with estimated shifts:"\
\[InvisibleSpace]{X0 \[Rule] \(\(4.741183395181137`\)\(\[InvisibleSpace]\)\) \
- 5.762936927109519`\ \[ImaginaryI], 
          X1 \[Rule] \(-0.04566741105406669`\) - 
              2.928942280103973`\ \[ImaginaryI], 
          X2 \[Rule] \(-5.906542046104681`\) + 0.`\ \[ImaginaryI], 
          X3 \[Rule] \(-0.04566741105406669`\) + 
              2.928942280103973`\ \[ImaginaryI], 
          X4 \[Rule] \(\(4.741183395181135`\)\(\[InvisibleSpace]\)\) + 
              5.7629369271095205`\ \[ImaginaryI]}\[InvisibleSpace]"\n"\
\[InvisibleSpace]"Coefficient error using estimated shifts: \
"\[InvisibleSpace]0.6553705003280953`\),
      SequenceForm[ "Exact coefficients:", {
        Complex[ 5, -1], 
        Complex[ 0, -3], -6, 
        Complex[ 0, 3], 
        Complex[ 5, 1]}, "\n\n", 
        "Estimated coefficients with exact offsets:", {
        Complex[ 4.7411833951811353, -0.53891110296805822], 
        Complex[ 0.23151318351691413, -2.9348596010982329], 
        Complex[ -5.945036752028237, -1.0375309099882165*^-16], 
        Complex[ 0.23151318351691505, 2.9348596010982324], 
        Complex[ 4.7411833951811353, 0.53891110296805822]}, "\n", 
        "Coefficient error using exact shifts: ", 0.079969464800000042, 
        "\n\n", "Estimated coefficients with estimated shifts:", {X0 -> 
        Complex[ 4.7411833951811371, -5.7629369271095188], X1 -> 
        Complex[ -0.045667411054066687, -2.9289422801039731], X2 -> 
        Complex[ -5.9065420461046809, 0.0], X3 -> 
        Complex[ -0.045667411054066687, 2.9289422801039731], X4 -> Complex[ 
        4.7411833951811353, 5.7629369271095205]}, "\n", 
        "Coefficient error using estimated shifts: ", 0.6553705003280953],
      Editable->False]], "Print",
  CellLabel->"From In[168]:="],

Cell[BoxData[
    \(1.9193543945890112`\)], "Print",
  CellLabel->"From In[168]:="]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 793}},
WindowSize->{699, 771},
WindowMargins->{{97, Automatic}, {Automatic, 0}},
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 222, 6, 144, "Title"],
Cell[1979, 59, 186, 4, 36, "Abstract"],
Cell[2168, 65, 683, 12, 114, "Copyright"],
Cell[2854, 79, 51, 0, 26, "Text"],

Cell[CellGroupData[{
Cell[2930, 83, 114, 1, 59, "Section"],
Cell[3047, 86, 96, 7, 107, "Input"],
Cell[3146, 95, 355, 7, 111, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3538, 107, 167, 3, 59, "Section"],

Cell[CellGroupData[{
Cell[3730, 114, 960, 21, 239, "Input"],
Cell[4693, 137, 125, 3, 73, "Output"],
Cell[4821, 142, 246, 5, 77, "Output"],
Cell[5070, 149, 129, 3, 60, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5248, 158, 78, 1, 59, "Section"],

Cell[CellGroupData[{
Cell[5351, 163, 1384, 27, 287, "Input"],
Cell[6738, 192, 576, 10, 112, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7363, 208, 134, 4, 59, "Section"],

Cell[CellGroupData[{
Cell[7522, 216, 628, 12, 127, "Input"],
Cell[8153, 230, 1020, 20, 132, "Output"],
Cell[9176, 252, 203, 4, 60, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9428, 262, 52, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[9505, 266, 296, 6, 79, "Input"],
Cell[9804, 274, 264, 5, 60, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10117, 285, 63, 0, 59, "Section"],
Cell[10183, 287, 767, 16, 175, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10987, 308, 82, 1, 59, "Section"],
Cell[11072, 311, 268, 5, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11377, 321, 84, 1, 59, "Section"],

Cell[CellGroupData[{
Cell[11486, 326, 106, 3, 43, "Subsubsection"],
Cell[11595, 331, 50, 0, 26, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11682, 336, 118, 3, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[11825, 343, 398, 9, 95, "Input"],
Cell[12226, 354, 1440, 23, 216, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13703, 382, 114, 2, 47, "Input"],
Cell[13820, 386, 4983, 82, 612, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18864, 475, 75, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[18964, 479, 394, 9, 111, "Input"],
Cell[19361, 490, 2384, 43, 206, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21794, 539, 103, 3, 59, "Section"],

Cell[CellGroupData[{
Cell[21922, 546, 520, 10, 143, "Input"],
Cell[22445, 558, 2067, 36, 152, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24549, 599, 443, 11, 111, "Input"],
Cell[24995, 612, 1567, 30, 166, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[26611, 648, 134, 3, 61, "Section"],
Cell[26748, 653, 179, 5, 82, "Text"],

Cell[CellGroupData[{
Cell[26952, 662, 1741, 33, 435, "Input"],
Cell[28696, 697, 347, 7, 76, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[29092, 710, 126, 3, 61, "Section"],

Cell[CellGroupData[{
Cell[29243, 717, 730, 15, 159, "Input"],
Cell[29976, 734, 382, 8, 23, "Print"],
Cell[30361, 744, 244, 5, 23, "Print"],
Cell[30608, 751, 146, 3, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30791, 759, 414, 10, 111, "Input"],
Cell[31208, 771, 530, 10, 76, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[31787, 787, 111, 3, 59, "Section"],

Cell[CellGroupData[{
Cell[31923, 794, 1786, 38, 455, "Input"],
Cell[33712, 834, 76, 2, 23, "Print"],
Cell[33791, 838, 421, 9, 39, "Print"],
Cell[34215, 849, 2743, 48, 169, "Print"],
Cell[36961, 899, 83, 2, 23, "Print"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

